@@ -25,7 +25,7 @@ BEGIN {
   );
 }
 
-our $VERSION = '2.003_005';
+our $VERSION = '2.004_001';
 $VERSION =~ tr/_//d;
 
 require Moo::sification;
@@ -63,38 +63,50 @@ sub _accessor_maker_for {
 
 sub _install_subs {
   my ($me, $target) = @_;
-  _install_tracked $target => has => sub {
-    my $name_proto = shift;
-    my @name_proto = ref $name_proto eq 'ARRAY' ? @$name_proto : $name_proto;
-    if (@_ % 2 != 0) {
-      croak("Invalid options for " . join(', ', map "'$_'", @name_proto)
-        . " attribute(s): even number of arguments expected, got " . scalar @_)
-    }
-    my %spec = @_;
-    foreach my $name (@name_proto) {
-      my $spec_ref = @name_proto > 1 ? +{%spec} : \%spec;
-      $me->_accessor_maker_for($target)
-        ->generate_method($target, $name, $spec_ref);
-      push @{$INFO{$target}{attributes}||=[]}, $name, $spec_ref;
-      $me->_maybe_reset_handlemoose($target);
-    }
-  };
-  # install before/after/around subs
-  foreach my $type (qw(before after around)) {
-    _install_tracked $target => $type => sub {
-      push @{$INFO{$target}{modifiers}||=[]}, [ $type => @_ ];
-      $me->_maybe_reset_handlemoose($target);
-    };
-  }
-  _install_tracked $target => requires => sub {
-    push @{$INFO{$target}{requires}||=[]}, @_;
-    $me->_maybe_reset_handlemoose($target);
-  };
-  _install_tracked $target => with => sub {
-    $me->apply_roles_to_package($target, @_);
-    $me->_maybe_reset_handlemoose($target);
-  };
+  my %install = $me->_gen_subs($target);
+  _install_tracked $target => $_ => $install{$_}
+    for sort keys %install;
   *{_getglob("${target}::meta")} = $me->can('meta');
+  return;
+}
+
+sub _gen_subs {
+  my ($me, $target) = @_;
+  return (
+    has => sub {
+      my $name_proto = shift;
+      my @name_proto = ref $name_proto eq 'ARRAY' ? @$name_proto : $name_proto;
+      if (@_ % 2 != 0) {
+        croak("Invalid options for " . join(', ', map "'$_'", @name_proto)
+          . " attribute(s): even number of arguments expected, got " . scalar @_)
+      }
+      my %spec = @_;
+      foreach my $name (@name_proto) {
+        my $spec_ref = @name_proto > 1 ? +{%spec} : \%spec;
+        $me->_accessor_maker_for($target)
+          ->generate_method($target, $name, $spec_ref);
+        push @{$INFO{$target}{attributes}||=[]}, $name, $spec_ref;
+        $me->_maybe_reset_handlemoose($target);
+      }
+    },
+    (map {
+      my $type = $_;
+      (
+        $type => sub {
+          push @{$INFO{$target}{modifiers}||=[]}, [ $type => @_ ];
+          $me->_maybe_reset_handlemoose($target);
+        },
+      )
+    } qw(before after around)),
+    requires => sub {
+      push @{$INFO{$target}{requires}||=[]}, @_;
+      $me->_maybe_reset_handlemoose($target);
+    },
+    with => sub {
+      $me->apply_roles_to_package($target, @_);
+      $me->_maybe_reset_handlemoose($target);
+    },
+  );
 }
 
 push @ON_ROLE_CREATE, sub {
@@ -345,12 +357,12 @@ sub apply_roles_to_object {
   my $class = ref $new;
   _set_loaded($class, (caller)[1]);
 
-  my $apply_defaults = exists $APPLY_DEFAULTS{$class} ? $APPLY_DEFAULTS{$class}
-    : $APPLY_DEFAULTS{$class} = do {
-    my %attrs = map { @{$INFO{$_}{attributes}||[]} } @roles;
+  my $apply_defaults = $APPLY_DEFAULTS{$class};
+  if (!defined $apply_defaults) {
+    my $attrs = { map @{$INFO{$_}{attributes}||[]}, @roles };
 
     if ($INC{'Moo.pm'}
-        and keys %attrs
+        and keys %$attrs
         and my $con_gen = Moo->_constructor_maker_for($class)
         and my $m = Moo->_accessor_maker_for($class)) {
 
@@ -374,11 +386,11 @@ sub apply_roles_to_object {
           else {
             ();
           }
-        } sort keys %attrs ),
+        } sort keys %$attrs ),
       );
       if ($code) {
         require Sub::Quote;
-        Sub::Quote::quote_sub(
+        $apply_defaults = Sub::Quote::quote_sub(
           "${class}::_apply_defaults",
           "no warnings 'void';\n$code",
           \%captures,
@@ -388,14 +400,10 @@ sub apply_roles_to_object {
           }
         );
       }
-      else {
-        0;
-      }
     }
-    else {
-      0;
-    }
-  };
+    $APPLY_DEFAULTS{$class} = $apply_defaults ||= 0;
+  }
+
   if ($apply_defaults) {
     local $Carp::Internal{+__PACKAGE__} = 1;
     local $Carp::Internal{$class} = 1;
@@ -497,7 +505,7 @@ And elsewhere:
   use strictures 2;
 
   # bar gets imported, but not foo
-  with('My::Role');
+  with 'My::Role';
 
   sub foo { ... }
 
